aboutsummaryrefslogtreecommitdiffstats
path: root/libsolidity/parsing/Parser.cpp
diff options
context:
space:
mode:
Diffstat (limited to 'libsolidity/parsing/Parser.cpp')
-rw-r--r--libsolidity/parsing/Parser.cpp22
1 files changed, 15 insertions, 7 deletions
diff --git a/libsolidity/parsing/Parser.cpp b/libsolidity/parsing/Parser.cpp
index 2b886121..caf38b1d 100644
--- a/libsolidity/parsing/Parser.cpp
+++ b/libsolidity/parsing/Parser.cpp
@@ -1035,27 +1035,35 @@ ASTPointer<Expression> Parser::parsePrimaryExpression()
expression = nodeFactory.createNode<Identifier>(getLiteralAndAdvance());
break;
case Token::LParen:
+ case Token::LBrack:
{
// Tuple or parenthesized expression.
// Special cases: () is empty tuple type, (x) is not a real tuple, (x,) is one-dimensional tuple
m_scanner->next();
vector<ASTPointer<Expression>> components;
- if (m_scanner->currentToken() != Token::RParen)
+ Token::Value oppositeToken = (token == Token::LParen ? Token::RParen : Token::RBrack);
+ bool isArray = (token == Token::LBrack);
+
+ if (m_scanner->currentToken() != oppositeToken)
while (true)
{
- if (m_scanner->currentToken() != Token::Comma && m_scanner->currentToken() != Token::RParen)
+ if (m_scanner->currentToken() != Token::Comma && m_scanner->currentToken() != oppositeToken)
components.push_back(parseExpression());
+ else if (isArray)
+ parserError("Expected expression (inline array elements cannot be omitted).");
else
components.push_back(ASTPointer<Expression>());
- if (m_scanner->currentToken() == Token::RParen)
+ if (m_scanner->currentToken() == oppositeToken)
break;
else if (m_scanner->currentToken() == Token::Comma)
m_scanner->next();
}
nodeFactory.markEndPosition();
- expectToken(Token::RParen);
- return nodeFactory.createNode<TupleExpression>(components);
+ expectToken(oppositeToken);
+ return nodeFactory.createNode<TupleExpression>(components, isArray);
}
+
default:
if (Token::isElementaryTypeName(token))
{
@@ -1035,27 +1035,35 @@ ASTPointer<Expression> Parser::parsePrimaryExpression()
{
- // Tuple or parenthesized expression.
- // Special cases: () is empty tuple type, (x) is not a real tuple, (x,) is one-dimensional tuple
+ // Tuple/parenthesized expression or inline array/bracketed expression.
+ // Special cases: ()/[] is empty tuple/array type, (x) is not a real tuple,
+ // (x,) is one-dimensional tuple, elements in arrays cannot be left out, only in tuples.
m_scanner->next();
vector<ASTPointer<Expression>> components;